define([], function () {

    const parseItem = (ansText) => {
        let value = null
        const parseFn = (ansText) => {
            switch (typeof ansText) {
                case "string":
                    value = `"${ansText.replace(/"/g, '\\"')}"`
                    break;
                case "function":
                    throw new Error('Invalid Answer: ans is a function')
                    break;
                default:
                    value = ansText
            }
        };

        const codeFunction = new Function('parseFn', `parseFn(${ansText})`);
        codeFunction(parseFn);
        return value;
    }

    const parseAnswerLine = (answerPart) => {
        const answerArray = convertAnswerLineToArray(answerPart);
        return answerArray.map(item => {
            if (!isValidItem(item)) {
                throw new Error(`Invalid Item: ${item}`);
            }
            return parseItem(item);
        });
    }

    const convertAnswerLineToArray = (answerLine) => {
        const regex = /^ln\((.*?)\)$/;
        const matches = answerLine.match(regex);
        return matches ? splitIntoArray(matches[1]) : [answerLine];
    }

    const isValidItem = (item) => {
        const invalidTypes = ['Function', 'Expression','Unknown', 'Symbol'];
        const type = analyzeInputRecursively(item);
        //console.log(item, type)
        if (invalidTypes.includes(type)) {
            return false;
        }

        if (type === 'Array') {
            const items = splitIntoArray(item.trim().slice(1, -1));
            for (const subItem of items) {
                if (typeof subItem === 'string' && !isValidItem(subItem)) {
                    return false;
                }
            }
        }

        if (type === 'Object') {
            let modifiedItem = item.trim().replace(/^\{/, '').replace(/\}$/, '');
            let objectValues = splitIntoArray(modifiedItem);
            for (let objectValue of objectValues) {
                let pair = objectValue.split(/:(.+)/);
                let value = pair[1].trim();
                if (!isValidItem(value.trim())) {
                    return false;
                }
            }
        }

        return true;
    }

    function splitIntoArray(input) {
        let blocks = [];
        let block = '';
        let bracketCount = 0;
        let inQuotes = false;
        let inSingleQuotes = false;
        let inEscapedQuotes = false;
        let previousChar = null;

        for (let i = 0; i < input.length; i++) {
            let char = input[i];
            if (char === '[' || char === '{') {
                bracketCount++;
            } else if (char === ']' || char === '}') {
                bracketCount--;
            } else if (char === '"' && previousChar !== '\\') {
                inQuotes = !inQuotes;
            } else if (char === "'" && previousChar !== '\\') {
                inSingleQuotes = !inSingleQuotes;
            }

            if (char === '\\' && input[i + 1] === '"') {
                inEscapedQuotes = true;
            } else {
                inEscapedQuotes = false;
            }

            if (char === ',' && bracketCount === 0 && !inQuotes && !inSingleQuotes && !inEscapedQuotes) {
                blocks.push(block.trim());
                block = '';
            } else {
                block += char;
            }

            previousChar = char;
        }

        if (block) {
            blocks.push(block.trim());
        }

        return blocks;
    }

    const analyzeInputRecursively = (text) => {
        const analyze = (text, depth) => {
            if (depth > 5) throw new Error("Recursion limit exceeded");

            const regexMap = {
                'Number': /^\s*[-+]?NaN$|^\s*[-+]?(?:\d*\.?\d+|\d+\.\d*)([eE][-+]?\d+)?\s*$/,
                'String': /^\s*(['"`]).*\1\s*$/,
                'Boolean': /^\s*(true|false)\s*$/,
                'null': /^\s*null\s*$/,
                'undefined': /^\s*undefined\s*$/,
                'Symbol': /^\s*Symbol\((.*)\)\s*$/,
                'Array': /^\s*\[.*\]\s*$/,
                'Object': /^\s*{.*}\s*$/,
                'Function': /^\s*function\s*\(.*\)\s*\{.*\}\s*$/,
                'Expression': /^\s*.*\(.*\)\s*$|^\s*\d+\s*[\+\-\*\/]\s*\d+\s*$/
            };

            for (const [type, regex] of Object.entries(regexMap)) {
                if (regex.test(text)) return type;
            }

            return 'Unknown';
        }

        return analyze(text, 1);
    }

    return function answerParser(answerInput) {
        let answerParts = [];
        try {
            for(let answerLine of answerInput?.split("\n")){
                if (answerLine === "") continue;
                const values = parseAnswerLine(answerLine);
                answerParts.push(values);
            }
        } catch(e) {
            console.log(e);
            return false;
        }
        return answerParts;
    }
});